import { Track } from 'livekit-client';
import { useEffect, useMemo, useState } from 'react';
import { addListener, removeListener } from '../events/EventEmitter';
import LiveKitModule from '../LKNativeModule';

/**
 * Interface for configuring options for the useMultibandTrackVolume hook.
 * @alpha
 */

const multibandDefaults = {
  bands: 5,
  minFrequency: 1000,
  maxFrequency: 8000,
  updateInterval: 40
};

/**
 * A hook for tracking the volume of an audio track across multiple frequency bands.
 *
 * @param trackOrTrackReference
 * @returns A number array containing the volume for each frequency band.
 */
export function useMultibandTrackVolume(trackOrTrackReference, options = {}) {
  var _trackOrTrackReferenc;
  const track = trackOrTrackReference instanceof Track ? trackOrTrackReference : trackOrTrackReference === null || trackOrTrackReference === void 0 || (_trackOrTrackReferenc = trackOrTrackReference.publication) === null || _trackOrTrackReferenc === void 0 ? void 0 : _trackOrTrackReferenc.track;
  const opts = useMemo(() => {
    return {
      ...multibandDefaults,
      ...options
    };
  }, [options]);
  const mediaStreamTrack = track === null || track === void 0 ? void 0 : track.mediaStreamTrack;
  let [magnitudes, setMagnitudes] = useState([]);
  useEffect(() => {
    let listener = Object();
    let reactTag = null;
    if (mediaStreamTrack) {
      reactTag = LiveKitModule.createMultibandVolumeProcessor(opts, mediaStreamTrack._peerConnectionId ?? -1, mediaStreamTrack.id);
      addListener(listener, 'LK_MULTIBAND_PROCESSED', event => {
        if (event.magnitudes && reactTag && event.id === reactTag) {
          console.log('event received: ', event.magnitudes[0]);
          setMagnitudes(event.magnitudes);
        }
      });
    }
    return () => {
      if (mediaStreamTrack) {
        removeListener(listener);
        if (reactTag) {
          LiveKitModule.deleteMultibandVolumeProcessor(reactTag, mediaStreamTrack._peerConnectionId ?? -1, mediaStreamTrack.id);
        }
      }
    };
  }, [mediaStreamTrack, opts]);
  return magnitudes;
}
//# sourceMappingURL=useMultibandTrackVolume.js.map